/* Konfabulator Animation Voodoo
   Provides custom animator objects.
   Author: Daniel Dickison (danieldickison@cmu.edu)
   Requires Konfabulator version 2.1.1+
   Nov. 25, 2005
*/

/***** DDAnimator ******
 * This is the parent class for animations.
 * Subclasses should implement onTick and, optionally, _onAnimationEnd in the
 * constructor.
 *
 * Public Methods:
 * - start()
 *     Starts the animation.
 * - stop(finalize)
 *     Stops the animation.  If finalize is true, runs the onAnimationEnd
 *     method and the subclass-specific finalization method (_onAnimationEnd).
 * - progress()
 *     Returns the percent [0, 1) of the animation that is complete.  This 
 *     should only be called after calling start() while running is true.
 *
 * Public Properties:
 * - fps: The (ideal) number of frames per second.
 * - interval: The number of ms between frames (1/fps).
 * - period: The time in ms to complete a cycle of the animation.
 * - perpetual: Whether the animation should cycle until stop() is called.
 * - easeType: One of the Konfabulator animation ease types.
 * - onAnimationEnd: A function that is called at the completion.
 * - running (read-only): Set to true when the animation is running.
 */

function DDAnimator() {
    this.objects = new Array();
}

DDAnimator.prototype.fps getter = function () {
    return 1000 / this.interval;
}
DDAnimator.prototype.fps setter = function (fps) {
    this.interval = 1000 / fps;
}

DDAnimator.prototype.onAnimationEnd = null;
DDAnimator.prototype._onAnimationEnd = null;
DDAnimator.prototype.easeType = animator.kEaseInOut;
DDAnimator.prototype.period = 1000;
DDAnimator.prototype.pepetual = false;
DDAnimator.prototype.interval = 50;
DDAnimator.prototype.running = false;

DDAnimator.prototype.start = function () {
    this.animation = this.makeAnimationObject();
    this.startTime = animator.milliseconds;
    this.running = true;
    animator.start(this.animation);
}

DDAnimator.prototype.stop = function (finalize) {
    if (this.animation) {
        this.animation.kill();
        //this.animation = null;
    }
    this.running = false;
    
    // Default to finalizing the animation when stopping.
    if (typeof finalize == "undefined")
        finalize = true;
        
    if (finalize) {
        if (this._onAnimationEnd)
            this._onAnimationEnd();
        if (this.onAnimationEnd) {
            if (typeof this.onAnimationEnd == "function")
                this.onAnimationEnd();
            else if (typeof this.onAnimationEnd == "string")
                eval(this.onAnimationEnd);
        }
    }
}

DDAnimator.prototype.makeAnimationObject = function () {
    var self = this;
    var updateFunc = function() {
        if (!self.running || (!self.perpetual && self.progress() >= 1.0)) {
            return false;
        }
        else {
            self.onTick();
            return true;
        }
    }
    var doneFunc = function () {
        self.stop();
    }
    return new CustomAnimation(this.interval, updateFunc, doneFunc);
}

DDAnimator.prototype.progress = function () {
    return (animator.milliseconds - this.startTime) / this.period;
}

DDAnimator.prototype.onTick = function () {
    // This method should be overloaded by subclasses.
    log("DDAnimator default tick executed");
}



/****** DDFadeAnimator *******
 * Fades an array of objects.
 *
 * Public Properties:
 * - startOpacity, endOpacity: duh.
 */

function DDFadeAnimator(startOpac, endOpac, period, perpetual, objects) {
    this.startOpacity = startOpac;
    this.endOpacity = endOpac;
    this.period = period;
    this.perpetual = perpetual;
    if (objects)
        this.objects = objects;
    var self = this;
    this._onAnimationEnd = function () {
        for (var i in self.objects) {
            self.objects[i].opacity = self.endOpacity;
        }
    }
    this.onTick = function () {
        var newOpac = animator.ease(self.startOpacity, self.endOpacity,
                                    self.progress(), self.easeType);
        for (var i in self.objects) {
            self.objects[i].opacity = newOpac;
        }
    }
}

DDFadeAnimator.prototype = new DDAnimator();

var _a
function testFade()
{
    _a = new DDFadeAnimator(255, 0, 1000, true, [playButton, artworkBg]);
    _a.start();
    _a.onAnimationEnd = "print('Done fading')";
}




/****** DDHSLAnimator ******/

function DDTintAnimator(startHSL, endHSL, period, perpetual, objects) {
    this.startHSL = startHSL;
    this.endHSL = endHSL;
    this.period = period;
    this.perpetual = perpetual;
    if (objects)
        this.objects = objects;
    var self = this;
    this._onAnimationEnd = function () {
        for (var i in self.objects) {
            self.objects[i].hslTinting = self.endHSL;
        }
    }
    this.onTick = function () {
        var percent = self.progress();
        var newH = animator.ease(self.startHSL[0], self.endHSL[0],
                                 percent, self.easeType);
        var newS = animator.ease(self.startHSL[1], self.endHSL[1],
                                 percent, self.easeType);
        var newL = animator.ease(self.startHSL[2], self.endHSL[2],
                                 percent, self.easeType);
        for (var i in self.objects) {
            self.objects[i].hslTinting = Math.floor(newH) + "," +
                                         Math.floor(newS) + "," +
                                         Math.floor(newL);
        }
    }
}

DDTintAnimator.prototype = new DDAnimator();

